@@ -37,6 +37,10 @@ class CopySpec(object):
     this is done be temporarily setting the attribute to a *ref_placeholder* value, performing the
     deep copy, and then (re)setting the attribute to the original object.
 
+    When *skip* (*skip_shallow*) is *True*, the attribute is not copied when the source objects is
+    copied through :py:meth:`CopyMixin.copy` (:py:meth:`CopyMixin.copy_shallow`). When skipped,
+    the attribute of the copied object will be *skip_value*.
+
     **Members**
 
     .. py:attribute:: dst
@@ -52,12 +56,28 @@ class CopySpec(object):
     .. py:attribute:: ref
        type: bool
 
-       Whether or not the attribute should be passed as a reference instead of copying.
+       Whether or not the attribute should be passed as a reference instead of being copied.
 
     .. py:attribute:: ref_placeholder
        type: any
 
-       Placeholder value that is used during the copying process.
+       Placeholder value for attributes carried over as a reference that is used during the copying
+       process.
+
+    .. py:attribute:: skip
+       type: bool
+
+       Whether or not the attribute is skipped when copied with :py:meth:`CopyMixin.copy`.
+
+    .. py:attribute:: skip_shallow
+       type: bool
+
+       Whether or not the attribute is skipped when copyied with :py:meth:`CopyMixin.copy_shallow`.
+
+    .. py:attribute:: skip_value
+       type: any
+
+       Value to be used for attributes that are skipped in the copy process.
     """
 
     @classmethod
@@ -78,13 +98,30 @@ class CopySpec(object):
                 msg = "cannot create {} from dict, a 'dst' or 'attr' field is required, got '{}'"
                 raise ValueError(msg.format(cls.__name__, obj))
             kwargs["src"] = obj.get("src", obj.get("attr"))
-            kwargs["ref"] = obj.get("ref", False)
-            kwargs["ref_placeholder"] = obj.get("ref_placeholder", None)
+            if "ref" in obj:
+                kwargs["ref"] = obj.get("ref", False)
+            if "ref_placeholder" in obj:
+                kwargs["ref_placeholder"] = obj.get("ref_placeholder", None)
+            if "skip" in obj:
+                kwargs["skip"] = obj.get("skip", False)
+            if "skip_shallow" in obj:
+                kwargs["skip_shallow"] = obj.get("skip_shallow", False)
+            if "skip_value" in obj:
+                kwargs["skip_value"] = obj.get("skip_value")
             return cls(**kwargs)
 
         raise TypeError("cannot create {} from object '{}'".format(cls.__name__, obj))
 
-    def __init__(self, dst, src=None, ref=False, ref_placeholder=None):
+    def __init__(
+        self,
+        dst,
+        src=None,
+        ref=False,
+        ref_placeholder=None,
+        skip=False,
+        skip_shallow=False,
+        skip_value=None,
+    ):
         super(CopySpec, self).__init__()
 
         # store attributes
@@ -92,6 +129,9 @@ class CopySpec(object):
         self.src = src or dst
         self.ref = ref
         self.ref_placeholder = ref_placeholder
+        self.skip = skip
+        self.skip_shallow = skip_shallow
+        self.skip_value = skip_value
 
     def __eq__(self, spec):
         """
@@ -123,7 +163,8 @@ class CopyMixin(object):
     .. note::
 
         At the moment, custom specs are only required for attributes that should not be copied but
-        carried over to the copied instance as a reference.
+        either carried over to the copied instance as a reference, or skipped in case only a shallow
+        copy is requested via :py:meth:`copy_shallow`.
 
     **Example**
 
@@ -137,17 +178,21 @@ class CopyMixin(object):
 
             copy_specs = [
                 {"attr": "obj", "ref": True},
+                {"attr": "complex_obj", "ref": True, "skip_shallow": True},
             ]
 
-            def __init__(self, name, obj):
+            def __init__(self, name, obj, complex_obj=None):
                 super(MyClass, self).__init__()
                 self.name = name
                 self.obj = obj
+                self.complex_obj = complex_obj
 
-        a = MyClass("foo", some_object)
+        a = MyClass("foo", some_object, "some_other_complex_object")
         a.name
         # -> "foo"
 
+        # normal copy
+
         b = a.copy()
         b.name
         # -> "foo"
@@ -155,6 +200,21 @@ class CopyMixin(object):
         b.obj is a.obj
         # -> True
 
+        b.complex_obj is a.complext_obj
+        # -> True
+
+        # shallow copy (skipping certain attributes)
+
+        c = a.copy_shallow()
+        c.name
+        # -> "foo"
+
+        c.obj is a.obj
+        # -> True
+
+        c.complex_obj is None  # note the None here
+        # -> True
+
     **Members**
 
     .. py:classattribute:: copy_specs
@@ -163,8 +223,29 @@ class CopyMixin(object):
        List of copy specifications per attribute.
     """
 
+    class Deferred(object):
+
+        def __init__(self, func):
+            self.func = func
+
+        def __call__(self, *args, **kwargs):
+            return self.func(*args, **kwargs)
+
     copy_spec = []
 
+    @classmethod
+    def _create_specs(cls, specs):
+        # ensure that specs contain CopySpec objects
+        # also remove duplicates, priotize last occurances
+        _specs = []
+
+        for spec in specs[::-1]:
+            spec = CopySpec.new(spec.__dict__ if isinstance(spec, CopySpec) else spec)
+            if spec not in _specs:
+                _specs.append(spec)
+
+        return _specs[::-1]
+
     def copy(self, *args, **kwargs):
         r"""copy(*args, **kwargs, _specs=None, _skip=None)
         Creates a copy of this instance and returns it. All *args* and *kwargs* are converted to
@@ -172,37 +253,46 @@ class CopyMixin(object):
         Additional specifications per attribute are taken from :py:attr:`copy_specs` or *_specs* if
         set. *_skip* can be a sequence of source attribute names that should be skipped.
         """
-        # extract the copy configuration from kwargs
-        specs = kwargs.pop("_specs", None) or self.copy_specs
-        skip = kwargs.pop("_skip", None) or []
+        # get CopySpec objects
+        specs = self._create_specs(kwargs.pop("_specs", None) or self.copy_specs)
 
-        # unite args and kwargs
-        kwargs.update(args_to_kwargs(self.__class__.__init__ if six.PY2 else self.__class__, args))
+        # apply additional skips
+        for src in (kwargs.pop("_skip", None) or []):
+            for spec in specs:
+                if spec.src == src:
+                    spec.skip = True
+                    break
 
-        # ensure that specs contain CopySpec objects
-        # also remove duplicates, prioritize last occurances
-        _specs = []
-        for spec in specs[::-1]:
-            if not isinstance(spec, CopySpec):
-                spec = CopySpec.new(spec)
-            if spec not in _specs:
-                _specs.append(spec)
-        specs = _specs[::-1]
-
-        # maybe skip some specs, identified by the source attribute
-        specs = list(filter((lambda spec: spec.src not in skip), specs))
-
-        # attributes that are not copied but carried over as a reference should be set to a
-        # placeholder first and reset later on
+        # attributes that are skipped or carried over as a reference should be set to a placeholder
+        # first and reset later on
+        skips = {}
         refs = {}
         for spec in specs:
-            if spec.ref:
+            if spec.skip:
+                skip_value = (
+                    spec.skip_value()
+                    if isinstance(spec.skip_value, self.Deferred) else
+                    spec.skip_value
+                )
+                skips[spec] = getattr(self, spec.src)
+                setattr(self, spec.src, skip_value)
+            elif spec.ref:
+                ref_placeholder = (
+                    spec.ref_placeholder()
+                    if isinstance(spec.ref_placeholder, self.Deferred) else
+                    spec.ref_placeholder
+                )
                 refs[spec] = getattr(self, spec.src)
-                setattr(self, spec.src, spec.ref_placeholder)
+                setattr(self, spec.src, ref_placeholder)
 
         # perform the deep copy operation
         inst = copy.deepcopy(self)
 
+        # reset skipped attributes
+        for spec, obj in skips.items():
+            # reset the old value of this instance
+            setattr(self, spec.src, obj)
+
         # reset references
         for spec, obj in refs.items():
             # reset the old value of this instance
@@ -210,12 +300,31 @@ class CopyMixin(object):
             # add a reference to the copied instance
             setattr(inst, spec.dst, obj)
 
+        # unite args and kwargs
+        kwargs.update(args_to_kwargs(self.__class__.__init__ if six.PY2 else self.__class__, args))
+
         # set additional attributes
         for attr, obj in kwargs.items():
             setattr(inst, attr, obj)
 
         return inst
 
+    def copy_shallow(self, *args, **kwargs):
+        r"""copy_shallow(*args, **kwargs, _specs=None, _skip=None)
+        Just like :py:meth:`copy`, creates a copy of this instance and returns it, however with all
+        attributes that were declared as *skip_shallow* skipped (see :py:class:`CopySpec`).
+        """
+        # get CopySpec objects
+        specs = self._create_specs(kwargs.pop("_specs", None) or self.copy_specs)
+
+        # use skip_shallow flags
+        for spec in specs:
+            spec.skip |= spec.skip_shallow
+
+        # use copy implementation
+        kwargs["_specs"] = specs
+        return self.copy(*args, **kwargs)
+
 
 class AuxDataMixin(object):
     """
@@ -486,6 +595,12 @@ class DataSourceMixin(object):
 
        The data source string for mc (``"mc"``).
 
+    .. py:classattribute:: allow_undefined_data_source
+       type: boolean
+
+       A configuration flag for this class that decides whether empty (*None*) data sources are
+       allowed. *False* by default.
+
     .. py:attribute:: is_data
        type: boolean
 
@@ -505,6 +620,9 @@ class DataSourceMixin(object):
     DATA_SOURCE_DATA = "data"
     DATA_SOURCE_MC = "mc"
 
+    # whether the data source is allowed to be undefined / None
+    allow_undefined_data_source = False
+
     copy_specs = []
 
     def __init__(self, is_data=False):
@@ -522,24 +640,31 @@ class DataSourceMixin(object):
 
     @is_data.setter
     def is_data(self, is_data):
-        if not isinstance(is_data, bool):
+        if is_data is None and self.allow_undefined_data_source:
+            self._is_data = None
+        elif isinstance(is_data, bool):
+            self._is_data = is_data
+        else:
             raise TypeError("invalid is_data type: {}".format(is_data))
 
-        self._is_data = is_data
-
     @property
     def is_mc(self):
-        return not self.is_data
+        return None if self.is_data is None else (not self.is_data)
 
     @is_mc.setter
     def is_mc(self, is_mc):
-        if not isinstance(is_mc, bool):
+        if is_mc is None and self.allow_undefined_data_source:
+            self._is_data = None
+        elif isinstance(is_mc, bool):
+            self._is_data = not is_mc
+        else:
             raise TypeError("invalid is_mc type: {}".format(is_mc))
 
-        self._is_data = not is_mc
-
     @property
     def data_source(self):
+        if self.is_data is None:
+            return None
+
         return self.DATA_SOURCE_DATA if self.is_data else self.DATA_SOURCE_MC
 
 
@@ -697,7 +822,7 @@ class SelectionMixin(object):
         """
         if not isinstance(self.selection, six.string_types):
             raise TypeError(
-                "cannot add selection expressions to existing non-string "
+                "cannot add selection expressions to existing non-string " +
                 "selection {}".format(self.selection),
             )
 
@@ -834,12 +959,16 @@ class LabelMixin(object):
 class ColorMixin(object):
     """
     Mixin-class that provides a color in terms of RGB values as well as some convenience methods.
+    Internally, up to three different color values are stored, accessible through attributes such as
+    :py:attr:`color1`, :py:attr:`color2` and :py:attr:`color3`. For convenience (and backwards
+    compatibility), omitting the number will forward to the primary color (1).
 
     **Arguments**
 
-    *color* can be a tuple of 3 or 4 numbers that are interpreted as red, green and blue color
-    values, and optinally an alpha value. Floats are stored internally. When integers are passed,
-    they are divided by 255.
+    Each *color* arguments can be a tuple of 3 or 4 numbers that are interpreted as red, green and
+    blue color values, and optinally an alpha value. Internally, all values are stored as floats, so
+    when integers are passed, they are divided by 255. Similar to the access, when *color1* is not
+    set but *color* is, the latter is interpreted as the former.
 
     **Example**
 
@@ -847,7 +976,7 @@ class ColorMixin(object):
 
         import order as od
 
-        c = od.ColorMixin(color=(255, 0.5, 100))
+        c = od.ColorMixin(color=(255, 0.5, 100), color2="#00f")
 
         c.color
         # -> (1.0, 0.5, 0.392)
@@ -858,165 +987,520 @@ class ColorMixin(object):
         c.color_alpha
         # -> 1.0
 
+        c.color2_int
+        # -> (0, 0, 255)
+
     **Members**
 
+    .. py:attribute:: color1_r
+       type: float
+
+       Red component of the primary color.
+
+    .. py:attribute:: color1_g
+       type: float
+
+       Green component of the primary color.
+
+    .. py:attribute:: color1_b
+       type: float
+
+       Blue component of the primary color.
+
+    .. py:attribute:: color1_r_int
+       type: int
+
+       Red component of the primary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color1_g_int
+       type: int
+
+       Green component of the primary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color1_b_int
+       type: int
+
+       Blue component of the primary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color1_alpha
+       type: float
+
+       The alpha value of the primary color, defaults to 1.0.
+
+    .. py:attribute:: color1
+       type: tuple (float)
+
+       The RGB values of the primary color in a 3-tuple.
+
+    .. py:attribute:: color1_int
+       type: tuple (int)
+
+       The RGB int values of the primary color in a 3-tuple.
+
+    .. py:attribute:: color2_r
+       type: float
+
+       Red component of the secondary color.
+
+    .. py:attribute:: color2_g
+       type: float
+
+       Green component of the secondary color.
+
+    .. py:attribute:: color2_b
+       type: float
+
+       Blue component of the secondary color.
+
+    .. py:attribute:: color2_r_int
+       type: int
+
+       Red component of the secondary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color2_g_int
+       type: int
+
+       Green component of the secondary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color2_b_int
+       type: int
+
+       Blue component of the secondary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color2_alpha
+       type: float
+
+       The alpha value of the secondary color, defaults to 1.0.
+
+    .. py:attribute:: color2
+       type: tuple (float)
+
+       The RGB values of the secondary color in a 3-tuple.
+
+    .. py:attribute:: color2_int
+       type: tuple (int)
+
+       The RGB int values of the secondary color in a 3-tuple.
+
+    .. py:attribute:: color3_r
+       type: float
+
+       Red component of the tertiary color.
+
+    .. py:attribute:: color3_g
+       type: float
+
+       Green component of the tertiary color.
+
+    .. py:attribute:: color3_b
+       type: float
+
+       Blue component of the tertiary color.
+
+    .. py:attribute:: color3_r_int
+       type: int
+
+       Red component of the tertiary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color3_g_int
+       type: int
+
+       Green component of the tertiary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color3_b_int
+       type: int
+
+       Blue component of the tertiary color, converted to an integer in the [0, 255] range.
+
+    .. py:attribute:: color3_alpha
+       type: float
+
+       The alpha value of the tertiary color, defaults to 1.0.
+
+    .. py:attribute:: color3
+       type: tuple (float)
+
+       The RGB values of the tertiary color in a 3-tuple.
+
+    .. py:attribute:: color3_int
+       type: tuple (int)
+
+       The RGB int values of the tertiary color in a 3-tuple.
+
     .. py:attribute:: color_r
        type: float
 
-       Red component.
+       Shorthand for :py:attr:`color1_r`.
 
     .. py:attribute:: color_g
        type: float
 
-       Green component.
+       Shorthand for :py:attr:`color1_g`.
 
     .. py:attribute:: color_b
        type: float
 
-       Blue component.
+       Shorthand for :py:attr:`color1_b`.
 
     .. py:attribute:: color_r_int
        type: int
 
-       Red component, converted to an integer in the [0, 255] range.
+       Shorthand for :py:attr:`color1_r_int`.
 
     .. py:attribute:: color_g_int
        type: int
 
-       Green component, converted to an integer in the [0, 255] range.
+       Shorthand for :py:attr:`color1_g_int`.
 
     .. py:attribute:: color_b_int
        type: int
 
-       Blue component, converted to an integer in the [0, 255] range.
+       Shorthand for :py:attr:`color1_b_int`.
 
     .. py:attribute:: color_alpha
        type: float
 
-       The alpha value, defaults to 1.0.
+       Shorthand for :py:attr:`color1_alpha`.
 
     .. py:attribute:: color
        type: tuple (float)
 
-       The RGB color values in a 3-tuple.
+       Shorthand for :py:attr:`color1`.
 
     .. py:attribute:: color_int
        type: tuple (int)
 
-       The RGB int color values in a 3-tuple.
+       Shorthand for :py:attr:`color1_int`.
     """
 
+    default_color = (0.0, 0.0, 0.0, 1.0)
+
     copy_specs = []
 
-    def __init__(self, color=None):
+    def __init__(self, color=None, color1=None, color2=None, color3=None):
         super(ColorMixin, self).__init__()
 
         # instance members
-        self._color_r = 0.0
-        self._color_g = 0.0
-        self._color_b = 0.0
-        self._color_alpha = 1.0
+        self._color1_set = False
+        self._color1_r, self._color1_g, self._color1_b, self._color1_alpha = self.default_color
+        self._color2_set = False
+        self._color2_r, self._color2_g, self._color2_b, self._color2_alpha = self.default_color
+        self._color3_set = False
+        self._color3_r, self._color3_g, self._color3_b, self._color3_alpha = self.default_color
 
         # set initial values
-        if color is not None:
-            self.color = color
+        if color1 is not None:
+            self.color1 = color1
+        elif color is not None:
+            self.color1 = color
+        if color2 is not None:
+            self.color2 = color2
+        if color3 is not None:
+            self.color3 = color3
 
-    @typed
-    def color_r(self, color_r):
-        if isinstance(color_r, six.integer_types):
-            if not (0 <= color_r <= 255):
-                raise ValueError("invalid color_r value: {}".format(color_r))
-            color_r /= 255.0
-        elif isinstance(color_r, float):
-            if not (0 <= color_r <= 1):
-                raise ValueError("invalid color_r value: {}".format(color_r))
+    @classmethod
+    def _parse_color_channel(cls, name, value):
+        if isinstance(value, six.integer_types):
+            if not (0 <= value <= 255):
+                raise ValueError("invalid {} value: {}".format(name, value))
+            value /= 255.0
+        elif isinstance(value, float):
+            if not (0 <= value <= 1):
+                raise ValueError("invalid {} value: {}".format(name, value))
         else:
-            raise TypeError("invalid color_r type: {}".format(color_r))
+            raise TypeError("invalid {} type: {}".format(name, value))
 
-        return color_r
+        return value
 
-    @typed
-    def color_g(self, color_g):
-        if isinstance(color_g, six.integer_types):
-            if not (0 <= color_g <= 255):
-                raise ValueError("invalid color_g value: {}".format(color_g))
-            color_g /= 255.0
-        elif isinstance(color_g, float):
-            if not (0 <= color_g <= 1):
-                raise ValueError("invalid color_g value: {}".format(color_g))
+    @classmethod
+    def _parse_color_alpha(cls, name, value):
+        if isinstance(value, (float, int)):
+            if not (0 <= value <= 1):
+                raise ValueError("invalid {} value: {}".format(name, value))
         else:
-            raise TypeError("invalid color_g type: {}".format(color_g))
+            raise TypeError("invalid {} type: {}".format(name, value))
 
-        return color_g
+        return float(value)
 
-    @typed
-    def color_b(self, color_b):
-        if isinstance(color_b, six.integer_types):
-            if not (0 <= color_b <= 255):
-                raise ValueError("invalid color_b value: {}".format(color_b))
-            color_b /= 255.0
-        elif isinstance(color_b, float):
-            if not (0 <= color_b <= 1):
-                raise ValueError("invalid color_b value: {}".format(color_b))
-        else:
-            raise TypeError("invalid color_b type: {}".format(color_b))
-
-        return color_b
-
-    @typed
-    def color_alpha(self, color_alpha):
-        if isinstance(color_alpha, (float, int)):
-            if not (0 <= color_alpha <= 1):
-                raise ValueError("invalid color_alpha value: {}".format(color_alpha))
-        else:
-            raise TypeError("invalid color_alpha type: {}".format(color_alpha))
-
-        return float(color_alpha)
-
-    @property
-    def color(self):
-        # color getter
-        return (self.color_r, self.color_g, self.color_b)
-
-    @color.setter
-    def color(self, color):
-        # color setter
-        if isinstance(color, six.string_types):
-            m = re.match(r"^\#([abcdef0-9]{3}|[abcdef0-9]{6})$", color.lower())
+    @classmethod
+    def _parse_color(cls, name, value):
+        if isinstance(value, six.string_types):
+            m = re.match(r"^\#([abcdef0-9]{3}|[abcdef0-9]{6})$", value.lower())
             if not m:
-                raise ValueError("invalid color value: {}".format(color))
+                raise ValueError("invalid {} value: {}".format(name, value))
             s = m.group(1)
             if len(s) == 3:
                 s = "".join(c1 + c2 for c1, c2 in zip(s, s))
-            color = [int(s[2 * i:2 * i + 2], base=16) for i in range(3)]
+            value = [int(s[2 * i:2 * i + 2], base=16) for i in range(3)]
 
-        if not isinstance(color, (tuple, list)):
-            raise TypeError("invalid color type: {}".format(color))
-        elif not len(color) in (3, 4):
-            raise ValueError("invalid color value: {}".format(color))
+        if not isinstance(value, (tuple, list)):
+            raise TypeError("invalid {} type: {}".format(name, value))
+        elif not len(value) in (3, 4):
+            raise ValueError("invalid {} value: {}".format(name, value))
 
-        self.color_r = color[0]
-        self.color_g = color[1]
-        self.color_b = color[2]
-        if len(color) == 4:
-            self.color_alpha = color[3]
+        return value
+
+    @classmethod
+    def _float_to_int(cls, value):
+        return min(255, max(0, int(round(value * 255))))
+
+    # primary color
 
     @property
-    def color_r_int(self):
-        # color_r_int getter
-        return min(255, max(0, int(round(self.color_r * 255))))
+    def color1_r(self):
+        return self._color1_r if self._color1_set else None
+
+    @color1_r.setter
+    def color1_r(self, color1_r):
+        self._color1_set = color1_r is not None
+        if self._color1_set:
+            self._color1_r = self._parse_color_channel("color1_r", color1_r)
+        else:
+            self.color1 = None
 
     @property
-    def color_g_int(self):
-        # color_g_int getter
-        return min(255, max(0, int(round(self.color_g * 255))))
+    def color1_g(self):
+        return self._color1_g if self._color1_set else None
+
+    @color1_g.setter
+    def color1_g(self, color1_g):
+        self._color1_set = color1_g is not None
+        if self._color1_set:
+            self._color1_g = self._parse_color_channel("color1_g", color1_g)
+        else:
+            self.color1 = None
 
     @property
-    def color_b_int(self):
-        # color_b_int getter
-        return min(255, max(0, int(round(self.color_b * 255))))
+    def color1_b(self):
+        return self._color1_b if self._color1_set else None
+
+    @color1_b.setter
+    def color1_b(self, color1_b):
+        self._color1_set = color1_b is not None
+        if self._color1_set:
+            self._color1_b = self._parse_color_channel("color1_b", color1_b)
+        else:
+            self.color1 = None
 
     @property
-    def color_int(self):
-        # color_int getter
-        return (self.color_r_int, self.color_g_int, self.color_b_int)
+    def color1_alpha(self):
+        return self._color1_alpha if self._color1_set else None
+
+    @color1_alpha.setter
+    def color1_alpha(self, color1_alpha):
+        self._color1_set = color1_alpha is not None
+        if self._color1_set:
+            self._color1_alpha = self._parse_color_channel("color1_alpha", color1_alpha)
+        else:
+            self.color1 = None
+
+    @property
+    def color1(self):
+        return (self.color1_r, self.color1_g, self.color1_b) if self._color1_set else None
+
+    @color1.setter
+    def color1(self, color1):
+        color1_set = color1 is not None
+        color1 = self._parse_color("color1", color1) if color1_set else self.default_color
+        self.color1_r, self.color1_g, self.color1_b = color1[:3]
+        if len(color1) == 4:
+            self._color1_alpha = color1[3]
+        self._color1_set = color1_set
+
+    @property
+    def color1_r_int(self):
+        return self._float_to_int(self.color1_r) if self._color1_set else None
+
+    @property
+    def color1_g_int(self):
+        return self._float_to_int(self.color1_g) if self._color1_set else None
+
+    @property
+    def color1_b_int(self):
+        return self._float_to_int(self.color1_b) if self._color1_set else None
+
+    @property
+    def color1_int(self):
+        return (
+            self.color1_r_int,
+            self.color1_g_int,
+            self.color1_b_int,
+        ) if self._color1_set else None
+
+    # primary color shorthands
+
+    color_r = color1_r
+    color_g = color1_g
+    color_b = color1_b
+    color_alpha = color1_alpha
+    color = color1
+    color_r_int = color1_r_int
+    color_g_int = color1_g_int
+    color_b_int = color1_b_int
+    color_int = color1_int
+
+    # secondary color
+
+    @property
+    def color2_r(self):
+        return self._color2_r if self._color2_set else None
+
+    @color2_r.setter
+    def color2_r(self, color2_r):
+        self._color2_set = color2_r is not None
+        if self._color2_set:
+            self._color2_r = self._parse_color_channel("color2_r", color2_r)
+        else:
+            self.color2 = None
+
+    @property
+    def color2_g(self):
+        return self._color2_g if self._color2_set else None
+
+    @color2_g.setter
+    def color2_g(self, color2_g):
+        self._color2_set = color2_g is not None
+        if self._color2_set:
+            self._color2_g = self._parse_color_channel("color2_g", color2_g)
+        else:
+            self.color2 = None
+
+    @property
+    def color2_b(self):
+        return self._color2_b if self._color2_set else None
+
+    @color2_b.setter
+    def color2_b(self, color2_b):
+        self._color2_set = color2_b is not None
+        if self._color2_set:
+            self._color2_b = self._parse_color_channel("color2_b", color2_b)
+        else:
+            self.color2 = None
+
+    @property
+    def color2_alpha(self):
+        return self._color2_alpha if self._color2_set else None
+
+    @color2_alpha.setter
+    def color2_alpha(self, color2_alpha):
+        self._color2_set = color2_alpha is not None
+        if self._color2_set:
+            self._color2_alpha = self._parse_color_channel("color2_alpha", color2_alpha)
+        else:
+            self.color2 = None
+
+    @property
+    def color2(self):
+        return (self.color2_r, self.color2_g, self.color2_b) if self._color2_set else None
+
+    @color2.setter
+    def color2(self, color2):
+        color2_set = color2 is not None
+        color2 = self._parse_color("color2", color2) if color2_set else self.default_color
+        self.color2_r, self.color2_g, self.color2_b = color2[:3]
+        if len(color2) == 4:
+            self._color2_alpha = color2[3]
+        self._color2_set = color2_set
+
+    @property
+    def color2_r_int(self):
+        return self._float_to_int(self.color2_r) if self._color2_set else None
+
+    @property
+    def color2_g_int(self):
+        return self._float_to_int(self.color2_g) if self._color2_set else None
+
+    @property
+    def color2_b_int(self):
+        return self._float_to_int(self.color2_b) if self._color2_set else None
+
+    @property
+    def color2_int(self):
+        return (
+            self.color2_r_int,
+            self.color2_g_int,
+            self.color2_b_int,
+        ) if self._color2_set else None
+
+    # tertiary color
+
+    @property
+    def color3_r(self):
+        return self._color3_r if self._color3_set else None
+
+    @color3_r.setter
+    def color3_r(self, color3_r):
+        self._color3_set = color3_r is not None
+        if self._color3_set:
+            self._color3_r = self._parse_color_channel("color3_r", color3_r)
+        else:
+            self.color3 = None
+
+    @property
+    def color3_g(self):
+        return self._color3_g if self._color3_set else None
+
+    @color3_g.setter
+    def color3_g(self, color3_g):
+        self._color3_set = color3_g is not None
+        if self._color3_set:
+            self._color3_g = self._parse_color_channel("color3_g", color3_g)
+        else:
+            self.color3 = None
+
+    @property
+    def color3_b(self):
+        return self._color3_b if self._color3_set else None
+
+    @color3_b.setter
+    def color3_b(self, color3_b):
+        self._color3_set = color3_b is not None
+        if self._color3_set:
+            self._color3_b = self._parse_color_channel("color3_b", color3_b)
+        else:
+            self.color3 = None
+
+    @property
+    def color3_alpha(self):
+        return self._color3_alpha if self._color3_set else None
+
+    @color3_alpha.setter
+    def color3_alpha(self, color3_alpha):
+        self._color3_set = color3_alpha is not None
+        if self._color3_set:
+            self._color3_alpha = self._parse_color_channel("color3_alpha", color3_alpha)
+        else:
+            self.color3 = None
+
+    @property
+    def color3(self):
+        return (self.color3_r, self.color3_g, self.color3_b) if self._color3_set else None
+
+    @color3.setter
+    def color3(self, color3):
+        color3_set = color3 is not None
+        color3 = self._parse_color("color3", color3) if color3_set else self.default_color
+        self.color3_r, self.color3_g, self.color3_b = color3[:3]
+        if len(color3) == 4:
+            self._color3_alpha = color3[3]
+        self._color3_set = color3_set
+
+    @property
+    def color3_r_int(self):
+        return self._float_to_int(self.color3_r) if self._color3_set else None
+
+    @property
+    def color3_g_int(self):
+        return self._float_to_int(self.color3_g) if self._color3_set else None
+
+    @property
+    def color3_b_int(self):
+        return self._float_to_int(self.color3_b) if self._color3_set else None
+
+    @property
+    def color3_int(self):
+        return (
+            self.color3_r_int,
+            self.color3_g_int,
+            self.color3_b_int,
+        ) if self._color3_set else None
