@@ -7,6 +7,7 @@ Tools to work with variables.
 
 __all__ = ["Variable"]
 
+import warnings
 
 import six
 
@@ -15,6 +16,13 @@ from order.mixins import CopyMixin, AuxDataMixin, TagMixin, SelectionMixin
 from order.util import ROOT_DEFAULT, typed, to_root_latex, make_list
 
 
+def _depr_attr(old, new):
+    warnings.warn(
+        "Variable attribute '{}' is deprecated; use '{}' instead'".format(old, new),
+        DeprecationWarning,
+    )
+
+
 class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
     r"""
     Class that provides simplified access to variables for convenience methods for plotting with
@@ -35,8 +43,14 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
 
     **Copy behavior**
 
+    ``copy()``
+
     All attributes are copied.
 
+    ``copy_shallow()``
+
+    No difference with respect to ``copy()``, all attributes are copied.
+
     **Example**
 
     .. code-block:: python
@@ -137,8 +151,7 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
     .. py:attribute:: x_labels
        type: list, None
 
-       A list of custom bin labels or *None*. When not *None*, its length must be the same as the
-       number of bins.
+       A list of custom bin labels or *None*.
 
     .. py:attribute:: x_labels_root
        type: list, None
@@ -163,16 +176,27 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
        A configurable NULL value for this variable, possibly denoting missing values. *None* is
        considered as "non-configured".
 
-    .. py:attribute:: log_x
+    .. py:attribute:: x_log
        type: boolean
 
        Whether or not the x-axis should be drawn logarithmically.
 
-    .. py:attribute:: log_y
+    .. py:attribute:: y_log
        type: boolean
 
        Whether or not the y-axis should be drawn logarithmically.
 
+    .. py:attribute:: x_discrete
+       type: boolean
+
+       Whether or not the x-axis is partitioned by discrete values (i.e, an integer axis). There is
+       not constraint on the :py:attr:`binning` setting, but it should be set accordingly.
+
+    .. py:attribute:: y_discrete
+       type: boolean
+
+       Whether or not the y-axis is partitioned by discrete values (i.e, an integer axis).
+
     .. py:attribute:: n_bins
        type: int
        read-only
@@ -226,8 +250,10 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         y_title="Entries",
         y_title_short=None,
         x_labels=None,
-        log_x=False,
-        log_y=False,
+        x_log=False,
+        y_log=False,
+        x_discrete=False,
+        y_discrete=False,
         unit="1",
         unit_format="{title} / {unit}",
         null_value=None,
@@ -235,6 +261,11 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         selection_mode=None,
         tags=None,
         aux=None,
+        # backwards compatibility
+        log_x=None,
+        log_y=None,
+        discrete_x=None,
+        discrete_y=None,
     ):
         UniqueObject.__init__(self, name, id)
         CopyMixin.__init__(self)
@@ -250,12 +281,28 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         self._y_title = None
         self._y_title_short = None
         self._x_labels = None
-        self._log_x = None
-        self._log_y = None
+        self._x_log = None
+        self._y_log = None
+        self._x_discrete = None
+        self._y_discrete = None
         self._unit = None
         self._unit_format = None
         self._null_value = None
 
+        # backwards compatibility for log and discrete flags
+        if log_x is not None:
+            _depr_attr("log_x", "x_log")
+            x_log = log_x
+        if log_y is not None:
+            _depr_attr("log_y", "y_log")
+            y_log = log_y
+        if discrete_x is not None:
+            _depr_attr("discrete_x", "x_discrete")
+            x_discrete = discrete_x
+        if discrete_y is not None:
+            _depr_attr("discrete_y", "y_discrete")
+            y_discrete = discrete_y
+
         # set initial values
         self.expression = expression
         self.binning = binning
@@ -264,8 +311,10 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         self.y_title = y_title
         self.y_title_short = y_title_short
         self.x_labels = x_labels
-        self.log_x = log_x
-        self.log_y = log_y
+        self.x_log = x_log
+        self.y_log = y_log
+        self.x_discrete = x_discrete
+        self.y_discrete = y_discrete
         self.unit = unit
         self.unit_format = unit_format
         self.null_value = null_value
@@ -302,13 +351,13 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         if isinstance(binning, list):
             if len(binning) < 2:
                 raise ValueError("minimum number of bin edges is 2: {}".format(binning))
-            elif not all(isinstance(b, bin_types) for b in binning):
+            if not all(isinstance(b, bin_types) for b in binning):
                 raise TypeError("invalid bin edge types: {}".format(binning))
             binning = [float(b) for b in binning]
         elif isinstance(binning, tuple):
             if len(binning) != 3:
                 raise ValueError("even binning must have length 3: {}".format(binning))
-            elif not all(isinstance(b, bin_types) for b in binning):
+            if not all(isinstance(b, bin_types) for b in binning):
                 raise TypeError("invalid binning types: {}".format(binning))
             binning = (int(binning[0]), float(binning[1]), float(binning[2]))
         else:
@@ -390,10 +439,8 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
     def x_labels(self, x_labels):
         if x_labels is None:
             return None
-        elif not isinstance(x_labels, (list, tuple)):
+        if not isinstance(x_labels, (list, tuple)):
             raise TypeError("invalid x_labels type: {}".format(x_labels))
-        elif len(x_labels) != self.n_bins:
-            raise ValueError("invalid x_labels length: {}".format(x_labels))
 
         return list(x_labels)
 
@@ -405,26 +452,42 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         return [to_root_latex(str(label)) for label in self.x_labels]
 
     @typed
-    def log_x(self, log_x):
-        # log_x parser
-        if not isinstance(log_x, bool):
-            raise TypeError("invalid log_x type: {}".format(log_x))
+    def x_log(self, x_log):
+        # x_log parser
+        if not isinstance(x_log, bool):
+            raise TypeError("invalid x_log type: {}".format(x_log))
 
-        return log_x
+        return x_log
 
     @typed
-    def log_y(self, log_y):
-        # log_y parser
-        if not isinstance(log_y, bool):
-            raise TypeError("invalid log_y type: {}".format(log_y))
+    def y_log(self, y_log):
+        # y_log parser
+        if not isinstance(y_log, bool):
+            raise TypeError("invalid y_log type: {}".format(y_log))
 
-        return log_y
+        return y_log
+
+    @typed
+    def x_discrete(self, x_discrete):
+        # x_discrete parser
+        if not isinstance(x_discrete, bool):
+            raise TypeError("invalid x_discrete type: {}".format(x_discrete))
+
+        return x_discrete
+
+    @typed
+    def y_discrete(self, y_discrete):
+        # y_discrete parser
+        if not isinstance(y_discrete, bool):
+            raise TypeError("invalid y_discrete type: {}".format(y_discrete))
+
+        return y_discrete
 
     @typed
     def unit(self, unit):
         if unit is None:
             return None
-        elif not isinstance(unit, six.string_types):
+        if not isinstance(unit, six.string_types):
             raise TypeError("invalid unit type: {}".format(unit))
 
         return str(unit)
@@ -481,25 +544,36 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         bin_width = self.bin_width
         return [self.x_min + i * bin_width for i in range(self.n_bins + 1)]
 
-    def get_full_x_title(self, short=False, root=ROOT_DEFAULT):
+    def get_full_x_title(self, unit=None, short=False, root=ROOT_DEFAULT):
         """
-        Returns the full title (i.e. with unit string) of the x-axis. When *short* is *True*, the
-        short version is returned. When *root* is *True*, the title is converted to *proper* ROOT
-        latex.
+        Returns the full title (i.e. with unit string) of the x-axis. When *unit* is *None*, it
+        defaults to the :py:attr:`unit` if this instance. No unit is shown if it is one or it
+        evaluates to *False*.
+
+        When *short* is *True*, the short version is returned. When *root* is *True*, the title is
+        converted to *proper* ROOT latex.
         """
         title = self.x_title_short if short else self.x_title
 
-        if self.unit not in (None, "1"):
-            title = self.unit_format.format(title=title, unit=self.unit)
+        # determine the unit
+        if unit is None:
+            unit = self.unit
+
+        # create the full title
+        if unit and unit not in ("1", 1):
+            title = self.unit_format.format(title=title, unit=unit)
 
         return to_root_latex(title) if root else title
 
-    def get_full_y_title(self, bin_width=None, short=False, root=ROOT_DEFAULT):
+    def get_full_y_title(self, bin_width=None, unit=None, short=False, root=ROOT_DEFAULT):
         """
         Returns the full title (i.e. with bin width and unit string) of the y-axis. When not *None*,
-        the value *bin_width* instead of the one evaluated from *binning* when even. When *short* is
-        *True*, the short version is returned. When *root* is *True*, the title is converted to
-        ROOT-style latex.
+        the value *bin_width* instead of the one evaluated from *binning* when even. When *unit* is
+        *None*, it defaults to the :py:attr:`unit` if this instance. No unit is shown if it is one
+        or it evaluates to *False*.
+
+        When *short* is *True*, the short version is returned. When *root* is *True*, the title is
+        converted to ROOT-style latex.
         """
         title = self.y_title_short if short else self.y_title
 
@@ -507,14 +581,20 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
         if bin_width is None and self.even_binning:
             bin_width = round(self.bin_width, 2)
 
+        # determine the unit
+        if unit is None:
+            unit = self.unit
+
         # add bin width and unit to the title
-        unit = []
-        if self.unit not in (None, "1"):
-            unit.append(str(self.unit))
+        parts = []
+        if unit and unit not in ("1", 1):
+            parts.append(str(unit))
         if bin_width:
-            unit.insert(0, str(bin_width))
-            if unit:
-                title = self.unit_format.format(title=title, unit=" ".join(unit))
+            parts.insert(0, str(bin_width))
+
+        # create the full title
+        if parts:
+            title = self.unit_format.format(title=title, unit=" ".join(parts))
 
         return to_root_latex(title) if root else title
 
@@ -558,7 +638,7 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
             "range": (self.x_min, self.x_max),
             "label": self.name,
         }
-        if self.log_x:
+        if self.x_log:
             data["log"] = True
 
         # update?
@@ -572,3 +652,45 @@ class Variable(UniqueObject, CopyMixin, AuxDataMixin, TagMixin, SelectionMixin):
                     del data[key]
 
         return data
+
+    # deprecated
+
+    @property
+    def log_x(self):
+        _depr_attr("log_x", "x_log")
+        return self.x_log
+
+    @log_x.setter
+    def log_x(self, log_x):
+        _depr_attr("log_x", "x_log")
+        self.x_log = log_x
+
+    @property
+    def log_y(self):
+        _depr_attr("log_y", "y_log")
+        return self.y_log
+
+    @log_y.setter
+    def log_y(self, log_y):
+        _depr_attr("log_y", "y_log")
+        self.y_log = log_y
+
+    @property
+    def discrete_x(self):
+        _depr_attr("discrete_x", "x_discrete")
+        return self.x_discrete
+
+    @discrete_x.setter
+    def discrete_x(self, discrete_x):
+        _depr_attr("discrete_x", "x_discrete")
+        self.x_discrete = discrete_x
+
+    @property
+    def discrete_y(self):
+        _depr_attr("discrete_y", "y_discrete")
+        return self.y_discrete
+
+    @discrete_y.setter
+    def discrete_y(self, discrete_y):
+        _depr_attr("discrete_y", "y_discrete")
+        self.y_discrete = discrete_y
